==> aif-2011.01-3.src.tar.gz: aif-as-2011.01/src/user/archserver/libs/lib-ui-as-interactive.sh <==
#!/bin/bash
#


# Adapt interactive_select_packages function from lib-ui-interactive.sh to take an argument list
# of repos that will be used for installation

# select_packages()
# prompts the user to select packages to install
#
# params: list of repos to use
# returns: 1 on error
interactive_select_packages() {

	# set up our install location if necessary and sync up so we can get package lists
	target_prepare_pacman $(echo "$@") || ( show_warning 'Pacman preparation failure' "Pacman preparation failed! Check $LOG for errors." && return 1 )

	repos=`list_pacman_repos target`
	notify "Package selection is split into two stages.  First you will select package groups that contain packages you may be interested in.  Then you will be presented with a full list of packages for each group, allowing you to fine-tune.\n\n
Note that right now the packages (and groups) selection is limited to the repos available at this time ($repos).  One you have your ArchServer system up and running, you have access to more repositories and packages."

    # show group listing for group selection, base is ON by default, all others are OFF
    local _grouplist="base ^ ON"
    for i in $(list_package_groups | sed "s/^base$/ /g"); do
        _grouplist="${_grouplist} ${i} - OFF"
    done

    ask_checklist "Select Package groups\nDO NOT deselect BASE unless you know what you're doing!" $_grouplist || return 1
    _grouplist=$ANSWER_CHECKLIST # _grouplist now contains all groups (the tags from the dialog checklist)

    # assemble a list of packages with groups, marking pre-selected ones
    # <package> <group> <selected>
    local _pkgtmp="$(list_packages repo $repos | awk '{print $2}')" # all packages in $repos
    local _pkglist=''

    which_group "$_pkgtmp"
    while read pkgname pkggroup; do
        # check if this package is in a selected group
        # slightly ugly but sorting later requires newlines in the variable
        if [ "${_grouplist/"\"$pkggroup\""/XXXX}" != "${_grouplist}" ]; then
            _pkglist="$(echo -e "${_pkglist}\n${pkgname} ${pkggroup} ON")"
        else
            _pkglist="$(echo -e "${_pkglist}\n${pkgname} ${pkggroup} OFF")"
        fi
    done <<< "$PACKAGE_GROUPS"

    # sort by group
    _pkglist="$(echo "$_pkglist" | sort -f -k 2)"
    [ -z "$_pkglist" ] && show_warning "No packages found" "Sorry. I could not find any packages. maybe your network is not setup correctly, you lost connection, no mirror setup, bad group, ..." && return 1

    ask_checklist "Select Packages To Install." $_pkglist || return 1
	var_TARGET_PACKAGES=$ANSWER_CHECKLIST # contains now all package names
    return 0
}

# target_prepare_pacman() is originally in lib-pacman.sh, it's overridden here
# to change the name of the repository to use.  The original code would not all
# any repos with a name other than "core".

# target_prepare_pacman() taken from setup. modified a bit
# configures pacman and syncs for the first time on destination system
#
# $@ repositories to enable (optional. default: core)
# returns: 1 on error
target_prepare_pacman() {   
	[ "$var_PKG_SOURCE_TYPE" = "cd" ] && local serverurl="${var_FILE_URL}"
	[ "$var_PKG_SOURCE_TYPE" = "net" ] && local serverurl="${var_SYNC_URL}"

	[ -z "$1" ] && repos=server-core
	[ -n "$1" ] && repos="$@"
	# Setup a pacman.conf in /tmp
	cat << EOF > /tmp/pacman.conf
[options]
CacheDir = ${var_TARGET_DIR}/var/cache/pacman/pkg
CacheDir = /src/core/pkg
Architecture = auto
EOF

for repo in $repos
do
		add_pacman_repo target ${repo} "Server = ${serverurl/\$repo/$repo}" # replace literal '$repo' in the serverurl string by "$repo" where $repo is our variable.
done
	# Set up the necessary directories for pacman use
	[ ! -d "${var_TARGET_DIR}/var/cache/pacman/pkg" ] && mkdir -m 755 -p "${var_TARGET_DIR}/var/cache/pacman/pkg"
	[ ! -d "${var_TARGET_DIR}/var/lib/pacman" ] && mkdir -m 755 -p "${var_TARGET_DIR}/var/lib/pacman"

	infofy "Refreshing package database..."
	$PACMAN_TARGET -Sy >$LOG 2>&1 || return 1 #TODO: make sure this also goes into the logfile. actually we should do this for many commands.
	return 0
}


# Originally in lib-ui-interactive.sh, edited to support kernel localversions and to change
# the name of the menu.lst entries to ArchServer
generate_grub_menulst() {
	local grubmenu="$var_TARGET_DIR/boot/grub/menu.lst"
	get_device_with_mount '/' || return 1
	local _rootpart=$ANSWER_DEVICE

	# Determine what is the device that acts as grub's root
	# This is the blockdevice where /boot lives, normally a seperate partition.
	#
	# Special handling: on md raid arrays
	# md raid could not work directly with grub. To get grub's root device we
	# parse the slave 0 in the md array to get a real blockdevice (/dev/sdXY)
	#
	# We get at last in grubdev the blockdevice in grub-legacy notation (ex: (hd0,0)
	#
    # We do may things double here and in interactive_grub_install
    # Better way would be to determine/ask neccassary things once and then
    # fill (and present) menu.lst to user. If user mean there is something
    # wrong with menu.lst settings (wrong boot device or wrong root=/device)
    # he better should re-run the interactive grub install and select correct
    # settings.
    debug FS "Grub _rootpart: ${_rootpart}"
    debug FS "Grub bootdev: "$bootdev
	# No seperate /boot partition
	if [ -z $bootdev ]; then
		# Special handling on md raid
		if [ $onraid ]; then
			grubdev=$(mapdev $(mdraid_slave0 ${_rootpart}))
            debug FS "onraid no sep boot slave0: "$(mdraid_slave0 ${_rootpart})
            debug FS "onraid no sep boot grubdev: "$grubdev
		else
			# No raid
			grubdev=$(mapdev ${_rootpart})
            debug FS "no sep boot grubdev: "$grubdev
		fi
		# Without seperate /boot partiton we have to specify this path
		subdir="/boot"
	# with seperate /boot partition
	else
		# Special handling on md raid
		if [ $onraid ]; then
			grubdev=$(mapdev $(mdraid_slave0 $bootdev))
            debug FS "onraid with sep boot slave0: "$(mdraid_slave0 $bootdev)
            debug FS "onraid with sep boot grubdev: "$grubdev
		else
			# No raid
			grubdev=$(mapdev $bootdev)
            debug FS "onraid with sep boot grubdev: "$grubdev
		fi
	fi
	# Now that we have our grub-legacy root device (grubdev).
    # keep the file from being completely bogus
            if [ "$grubdev" = "DEVICE NOT FOUND" ]; then
                notify "Your root boot device could not be autodetected by setup.  Ensure you adjust the 'root (hd0,0)' line in your GRUB config accordingly."
                grubdev="(hd0,0)"
            fi
            # remove default entries by truncating file at our little tag (#-*)
            sed -i -e '/#-\*/q' $grubmenu

        # attempt to use a UUID if the root device has one
        local _uuid="$(getuuid ${_rootpart})"
        if [ -n "${_uuid}" ]; then
            _rootpart="/dev/disk/by-uuid/${_uuid}"
        fi

		# handle dmraid/mdadm,lvm,dm_crypt etc. replace entries where needed automatically
		kernel="kernel $subdir/vmlinuz26${localversion} root=${_rootpart} ro"
		if get_anchestors_mount ';/;'
		then
			if echo "$ANSWER_DEVICES" | sed -n '1p' | grep -q 'dm_crypt$' && echo "$ANSWER_DEVICES" | sed -n '2p' | grep -q 'raw$'
			then
				debug 'FS' 'Grub kernel line? Found / on dm_crypt on raw'
				raw_device=`echo "$ANSWER_DEVICES" | sed -n '2p' | cut -d ' ' -f1`
				kernel="kernel $subdir/vmlinuz26${localversion} root=$raw_device ro"
			elif echo "$ANSWER_DEVICES" | sed -n '1p' | grep -q 'lvm-lv$' && echo "$ANSWER_DEVICES" | sed -n '4p' | grep -q 'dm_crypt$' && echo "$ANSWER_DEVICES" | sed -n '5p' | grep -q 'raw$'
			then
				debug 'FS' 'Grub kernel line? Found / on lvm on dm_crypt on raw'
				lv_device=`echo "$ANSWER_DEVICES" | sed -n '1p' | cut -d ' ' -f1`
				vg_device=`echo "$ANSWER_DEVICES" | sed -n '2p' | cut -d ' ' -f1`
				raw_device=`echo "$ANSWER_DEVICES" | sed -n '5p' | cut -d ' ' -f1`
				kernel="kernel $subdir/vmlinuz26${localversion} root=$lv_device cryptdevice=$raw_device:`basename $vg_device` ro"
			elif echo "$ANSWER_DEVICES" | sed -n '1p' | grep -q 'dm_crypt$' && echo "$ANSWER_DEVICES" | sed -n '2p' | grep -q 'lvm-lv$' && echo "$ANSWER_DEVICES" | sed -n '5p' | grep -q 'raw$'
			then
				debug 'FS' 'Grub kernel line? Found / on dm_crypt on lvm on raw'
				crypt_device=`echo "$ANSWER_DEVICES" | sed -n '1p' | cut -d ' ' -f1`
				lv_device=`echo "$ANSWER_DEVICES" | sed -n '2p' | cut -d ' ' -f1`
				kernel=" kernel $subdir/vmlinuz26${localversion} root=$crypt_device cryptdevice=$lv_device:root ro"
			elif echo "$ANSWER_DEVICES" | sed -n '1p' | grep -q 'raw$'
			then
				debug 'FS' 'Grub kernel line? Found / on raw'
			elif echo "$ANSWER_DEVICES" | sed -n '1p' | grep -q 'lvm-lv$' && echo "$ANSWER_DEVICES" | sed -n '4p' | grep -q 'raw$'
			then
				debug 'FS' 'Grub kernel line? Found / on lvm on raw'
			else
				debug 'FS' 'Grub kernel line? Could not figure this one out'
				show_warning "Disk setup detection" "Are you using some really fancy dm_crypt/lvm/softraid setup?\nI could not figure it out, so the kernel line will be the default: you'll probably need to edit it.\nPlease file a bug for this and supply all files from /tmp/aif/"
			fi
		else
			show_warning "Disk setup detection" "Could not find out where your / is.  Did you setup filesystems/blockdevices? manual/autoprepare?  If yes, please file a bug and tell us about this"
		fi
            cat >>$grubmenu <<EOF

# (0) ArchServer
title  ArchServer
root   $grubdev
$kernel
initrd $subdir/kernel26${localversion}.img

# (1) ArchServer
title  ArchServer Fallback
root   $grubdev
$kernel
initrd $subdir/kernel26${localversion}-fallback.img
EOF
}

# run_mkinitcpio is in lib-software.sh.  It's over-ridden here to allow different kernel localversions

# run_mkinitcpio() taken from setup. adapted a lot
# runs mkinitcpio on the target system, displays output
run_mkinitcpio()  
{
	target_special_fs on

	run_controlled mkinitcpio "chroot $var_TARGET_DIR /sbin/mkinitcpio -p kernel26${localversion}" $TMP_MKINITCPIO_LOG "Rebuilding initcpio images ..."

	target_special_fs off

	# alert the user to fatal errors
	[ $CONTROLLED_EXIT -ne 0 ] && show_warning "MKINITCPIO FAILED - SYSTEM MAY NOT BOOT" "$TMP_MKINITCPIO_LOG" text
	return $CONTROLLED_EXIT
}

==> aif-2011.02-1.src.tar.gz: aif-as-2011.02/src/user/archserver/libs/lib-ui-as-interactive.sh <==
#!/bin/bash
#


# Adapt interactive_select_packages function from lib-ui-interactive.sh to take an argument list
# of repos that will be used for installation

# select_packages()
# prompts the user to select packages to install
#
# params: list of repos to use
# returns: 1 on error
interactive_select_packages() {

	# set up our install location if necessary and sync up so we can get package lists
	target_prepare_pacman $(echo "$@") || ( show_warning 'Pacman preparation failure' "Pacman preparation failed! Check $LOG for errors." && return 1 )

	repos=`list_pacman_repos target`
	notify "Package selection is split into two stages.  First you will select package groups that contain packages you may be interested in.  Then you will be presented with a full list of packages for each group, allowing you to fine-tune.\n\n
Note that right now the packages (and groups) selection is limited to the repos available at this time ($repos).  One you have your ArchServer system up and running, you have access to more repositories and packages."

    # show group listing for group selection, base is ON by default, all others are OFF
    local _grouplist="base ^ ON"
    for i in $(list_package_groups | sed "s/^base$/ /g"); do
        _grouplist="${_grouplist} ${i} - OFF"
    done

    ask_checklist "Select Package groups\nDO NOT deselect BASE unless you know what you're doing!" $_grouplist || return 1
    _grouplist=$ANSWER_CHECKLIST # _grouplist now contains all groups (the tags from the dialog checklist)

    # assemble a list of packages with groups, marking pre-selected ones
    # <package> <group> <selected>
    local _pkgtmp="$(list_packages repo $repos | awk '{print $2}')" # all packages in $repos
    local _pkglist=''

    which_group "$_pkgtmp"
    while read pkgname pkggroup; do
        # check if this package is in a selected group
        # slightly ugly but sorting later requires newlines in the variable
        if [ "${_grouplist/"\"$pkggroup\""/XXXX}" != "${_grouplist}" ]; then
            _pkglist="$(echo -e "${_pkglist}\n${pkgname} ${pkggroup} ON")"
        else
            _pkglist="$(echo -e "${_pkglist}\n${pkgname} ${pkggroup} OFF")"
        fi
    done <<< "$PACKAGE_GROUPS"

    # sort by group
    _pkglist="$(echo "$_pkglist" | sort -f -k 2)"
    [ -z "$_pkglist" ] && show_warning "No packages found" "Sorry. I could not find any packages. maybe your network is not setup correctly, you lost connection, no mirror setup, bad group, ..." && return 1

    ask_checklist "Select Packages To Install." $_pkglist || return 1
	var_TARGET_PACKAGES=$ANSWER_CHECKLIST # contains now all package names
    return 0
}

# target_prepare_pacman() is originally in lib-pacman.sh, it's overridden here
# to change the name of the repository to use.  The original code would not all
# any repos with a name other than "core".

# target_prepare_pacman() taken from setup. modified a bit
# configures pacman and syncs for the first time on destination system
#
# $@ repositories to enable (optional. default: core)
# returns: 1 on error
target_prepare_pacman() {   
	[ "$var_PKG_SOURCE_TYPE" = "cd" ] && local serverurl="${var_FILE_URL}"
	[ "$var_PKG_SOURCE_TYPE" = "net" ] && local serverurl="${var_SYNC_URL}"

	[ -z "$1" ] && repos=server-core
	[ -n "$1" ] && repos="$@"
	# Setup a pacman.conf in /tmp
	cat << EOF > /tmp/pacman.conf
[options]
CacheDir = ${var_TARGET_DIR}/var/cache/pacman/pkg
CacheDir = /src/core/pkg
Architecture = auto
EOF

for repo in $repos
do
		add_pacman_repo target ${repo} "Server = ${serverurl/\$repo/$repo}" # replace literal '$repo' in the serverurl string by "$repo" where $repo is our variable.
done
	# Set up the necessary directories for pacman use
	[ ! -d "${var_TARGET_DIR}/var/cache/pacman/pkg" ] && mkdir -m 755 -p "${var_TARGET_DIR}/var/cache/pacman/pkg"
	[ ! -d "${var_TARGET_DIR}/var/lib/pacman" ] && mkdir -m 755 -p "${var_TARGET_DIR}/var/lib/pacman"

	inform "Refreshing package database..."
	$PACMAN_TARGET -Sy >$LOG 2>&1 || return 1 #TODO: make sure this also goes into the logfile. actually we should do this for many commands.
	return 0
}


# Originally in lib-ui-interactive.sh, edited to support kernel localversions and to change
# the name of the menu.lst entries to ArchServer
generate_grub_menulst() {
	# Check if the user booted in runlevel 4 which would indicate they are
	# installing over a serial console
	local cur_runlevel=`runlevel | sed 's/.* \(.*\)/\1/'`
	if [ $cur_runlevel -eq 4 ]
		ask_yesno "You appear to have booted with serial console support.\nAnswer 'YES' to configure the system for login over serial console." no
		if [ $? -eq 0 ]; then
			local serialconsole=true
			local sc_line1="serial --unit=0 --speed=9600 --word=8 --parity=no --stop=1"
			local sc_line2="terminal serial"
			local serial_console="console=tty console=ttyS0,9600n8 4"
		else
			local serialconsole=false
		fi
	fi

	local grubmenu="$var_TARGET_DIR/boot/grub/menu.lst"
	get_device_with_mount '/' || return 1
	local _rootpart=$ANSWER_DEVICE

	# Determine what is the device that acts as grub's root
	# This is the blockdevice where /boot lives, normally a seperate partition.
	#
	# Special handling: on md raid arrays
	# md raid could not work directly with grub. To get grub's root device we
	# parse the slave 0 in the md array to get a real blockdevice (/dev/sdXY)
	#
	# We get at last in grubdev the blockdevice in grub-legacy notation (ex: (hd0,0)
	#
    # We do may things double here and in interactive_grub_install
    # Better way would be to determine/ask neccassary things once and then
    # fill (and present) menu.lst to user. If user mean there is something
    # wrong with menu.lst settings (wrong boot device or wrong root=/device)
    # he better should re-run the interactive grub install and select correct
    # settings.
    debug FS "Grub _rootpart: ${_rootpart}"
    debug FS "Grub bootdev: "$bootdev
	# No seperate /boot partition
	if [ -z $bootdev ]; then
		# Special handling on md raid
		if [ $onraid ]; then
			grubdev=$(mapdev $(mdraid_slave0 ${_rootpart}))
            debug FS "onraid no sep boot slave0: "$(mdraid_slave0 ${_rootpart})
            debug FS "onraid no sep boot grubdev: "$grubdev
		else
			# No raid
			grubdev=$(mapdev ${_rootpart})
            debug FS "no sep boot grubdev: "$grubdev
		fi
		# Without seperate /boot partiton we have to specify this path
		subdir="/boot"
	# with seperate /boot partition
	else
		# Special handling on md raid
		if [ $onraid ]; then
			grubdev=$(mapdev $(mdraid_slave0 $bootdev))
            debug FS "onraid with sep boot slave0: "$(mdraid_slave0 $bootdev)
            debug FS "onraid with sep boot grubdev: "$grubdev
		else
			# No raid
			grubdev=$(mapdev $bootdev)
            debug FS "onraid with sep boot grubdev: "$grubdev
		fi
	fi
	# Now that we have our grub-legacy root device (grubdev).
    # keep the file from being completely bogus
            if [ "$grubdev" = "DEVICE NOT FOUND" ]; then
                notify "Your root boot device could not be autodetected by setup.  Ensure you adjust the 'root (hd0,0)' line in your GRUB config accordingly."
                grubdev="(hd0,0)"
            fi
            # remove default entries by truncating file at our little tag (#-*)
            sed -i -e '/#-\*/q' $grubmenu

        # attempt to use a UUID if the root device has one
        local _uuid="$(getuuid ${_rootpart})"
        if [ -n "${_uuid}" ]; then
            _rootpart="/dev/disk/by-uuid/${_uuid}"
        fi

		# handle dmraid/mdadm,lvm,dm_crypt etc. replace entries where needed automatically
		kernel="kernel $subdir/vmlinuz26${localversion} root=${_rootpart} ro $serial_console"
		if get_anchestors_mount ';/;'
		then
			if echo "$ANSWER_DEVICES" | sed -n '1p' | grep -q 'dm_crypt$' && echo "$ANSWER_DEVICES" | sed -n '2p' | grep -q 'raw$'
			then
				debug 'FS' 'Grub kernel line? Found / on dm_crypt on raw'
				raw_device=`echo "$ANSWER_DEVICES" | sed -n '2p' | cut -d ' ' -f1`
				kernel="kernel $subdir/vmlinuz26${localversion} root=$raw_device ro $serial_console"
			elif echo "$ANSWER_DEVICES" | sed -n '1p' | grep -q 'lvm-lv$' && echo "$ANSWER_DEVICES" | sed -n '4p' | grep -q 'dm_crypt$' && echo "$ANSWER_DEVICES" | sed -n '5p' | grep -q 'raw$'
			then
				debug 'FS' 'Grub kernel line? Found / on lvm on dm_crypt on raw'
				lv_device=`echo "$ANSWER_DEVICES" | sed -n '1p' | cut -d ' ' -f1`
				vg_device=`echo "$ANSWER_DEVICES" | sed -n '2p' | cut -d ' ' -f1`
				raw_device=`echo "$ANSWER_DEVICES" | sed -n '5p' | cut -d ' ' -f1`
				kernel="kernel $subdir/vmlinuz26${localversion} root=$lv_device cryptdevice=$raw_device:`basename $vg_device` ro $serial_console"
			elif echo "$ANSWER_DEVICES" | sed -n '1p' | grep -q 'dm_crypt$' && echo "$ANSWER_DEVICES" | sed -n '2p' | grep -q 'lvm-lv$' && echo "$ANSWER_DEVICES" | sed -n '5p' | grep -q 'raw$'
			then
				debug 'FS' 'Grub kernel line? Found / on dm_crypt on lvm on raw'
				crypt_device=`echo "$ANSWER_DEVICES" | sed -n '1p' | cut -d ' ' -f1`
				lv_device=`echo "$ANSWER_DEVICES" | sed -n '2p' | cut -d ' ' -f1`
				kernel=" kernel $subdir/vmlinuz26${localversion} root=$crypt_device cryptdevice=$lv_device:root ro $serial_console"
			elif echo "$ANSWER_DEVICES" | sed -n '1p' | grep -q 'raw$'
			then
				debug 'FS' 'Grub kernel line? Found / on raw'
			elif echo "$ANSWER_DEVICES" | sed -n '1p' | grep -q 'lvm-lv$' && echo "$ANSWER_DEVICES" | sed -n '4p' | grep -q 'raw$'
			then
				debug 'FS' 'Grub kernel line? Found / on lvm on raw'
			else
				debug 'FS' 'Grub kernel line? Could not figure this one out'
				show_warning "Disk setup detection" "Are you using some really fancy dm_crypt/lvm/softraid setup?\nI could not figure it out, so the kernel line will be the default: you'll probably need to edit it.\nPlease file a bug for this and supply all files from /tmp/aif/"
			fi
		else
			show_warning "Disk setup detection" "Could not find out where your / is.  Did you setup filesystems/blockdevices? manual/autoprepare?  If yes, please file a bug and tell us about this"
		fi
            cat >>$grubmenu <<EOF
$sc_line1
$sc_line2

# (0) ArchServer
title  ArchServer
root   $grubdev
$kernel
initrd $subdir/kernel26${localversion}.img

# (1) ArchServer
title  ArchServer Fallback
root   $grubdev
$kernel
initrd $subdir/kernel26${localversion}-fallback.img
EOF
}

# run_mkinitcpio is in lib-software.sh.  It's over-ridden here to allow different kernel localversions

# run_mkinitcpio() taken from setup. adapted a lot
# runs mkinitcpio on the target system, displays output
run_mkinitcpio()  
{
	target_special_fs on

	run_controlled mkinitcpio "chroot $var_TARGET_DIR /sbin/mkinitcpio -p kernel26${localversion}" $TMP_MKINITCPIO_LOG "Rebuilding initcpio images ..."

	target_special_fs off

	# alert the user to fatal errors
	[ $CONTROLLED_EXIT -ne 0 ] && show_warning "MKINITCPIO FAILED - SYSTEM MAY NOT BOOT" "$TMP_MKINITCPIO_LOG" text
	return $CONTROLLED_EXIT
}
